@@ -5,12 +5,10 @@ from .vision.learner import ImageClassLearner
 from .text.learner import BERTTextClassLearner, TransformerTextClassLearner
 from .text.ner.learner import NERLearner
 from .graph.learner import NodeClassLearner, LinkPredLearner
-from .data import Dataset, TFDataset, SequenceDataset
 
 from . import utils as U
 
-__all__ = ['get_learner', 'get_predictor', 'load_predictor', 'release_gpu_memory',
-           'Dataset', 'TFDataset', 'SequenceDataset']
+__all__ = ['get_learner', 'get_predictor', 'load_predictor', 'release_gpu_memory' ]
 
 
 
@@ -47,7 +45,7 @@ def get_learner(model, train_data=None, val_data=None,
     """
 
     # check arguments
-    if not isinstance(model, I.Model):
+    if not isinstance(model, I.keras.Model):
         raise ValueError('model must be of instance Model')
     U.data_arg_check(train_data=train_data, val_data=val_data)
     if type(workers) != type(1) or workers < 1:
@@ -119,3 +117,9 @@ def get_learner(model, train_data=None, val_data=None,
 # keys
 # currently_unsupported: unsupported or disabled features (e.g., xai graph neural networks have not been implemented)
 # dep_fix:  a fix to address a problem in a dependency
+# TODO: things to change
+
+
+# NOTES: As of 0.30.x, TensorFlow is optional and no longer forced to allow for use of pretrained PyTorch or sklearn models.
+# In core, lroptimize imports were localized to allow for optional TF
+# References to ktrain.dataset (keras.utils) and anago (keras.Callback) were also localized (from module-level) for optional TF
