@@ -1,9 +1,6 @@
 from .imports import *
 
-from .lroptimize.sgdr import *
-from .lroptimize.triangular import *
 from .lroptimize.lrfinder import *
-from .lroptimize.optimization import AdamWeightDecay
 from . import utils as U
 
 from .vision.preprocessor import ImagePreprocessor
@@ -27,8 +24,8 @@ class Learner(ABC):
 
     """
     def __init__(self, model, workers=1, use_multiprocessing=False):
-        if not isinstance(model, Model):
-            raise ValueError('model must be of instance Model')
+        if not isinstance(model, keras.Model):
+            raise ValueError('model must be of instance keras.Model')
         self.model = model
         self.lr_finder = LRFinder(self.model)
         self.workers = workers
@@ -395,8 +392,8 @@ class Learner(ABC):
         replace model in this Learner instance
         ```
         """
-        if not isinstance(model, Model):
-            raise ValueError('model must be of instance Model')
+        if not isinstance(model, keras.Model):
+            raise ValueError('model must be of instance keras.Model')
         self.model = model
         self.history = None
         return
@@ -424,7 +421,7 @@ class Learner(ABC):
             # first find last dense layer
             dense_id = None
             for i, layer in reversed(list(enumerate(self.model.layers))):
-                if isinstance(layer, Dense):
+                if isinstance(layer, keras.layers.Dense):
                     dense_id = i
                     break
             if dense_id is None: raise Exception('cannot find Dense layer in this model')
@@ -599,7 +596,7 @@ class Learner(ABC):
         U.vprint('done.', verbose=verbose)
         if show_plot:
             U.vprint('Visually inspect loss plot and select learning rate associated with falling loss', verbose=verbose)
-            self.lr_plot()
+            self.lr_plot(suggest=suggest)
         else:
             U.vprint('Please invoke the Learner.lr_plot() method to visually inspect '
                   'the loss plot to help identify the maximal learning rate '
@@ -658,7 +655,9 @@ class Learner(ABC):
         ```
         plots training history
         Args:
-          plot_type (str):  one of {'loss', 'lr', 'momentum'}
+          plot_type (str):  A valid value in tf.keras History.  Either a built-in value  {'loss', 'lr', 'momentum'} or
+                            other values previously specified by user.  For instance, if 'mae' and/or 'mse' is previously specified as metrics
+                            when creating model, then these values can also be specified.
           return_fig(bool):  If True, return matplotlib.figure.Figure
         Return:
           matplotlib.figure.Figure if return_fig else None
@@ -666,6 +665,8 @@ class Learner(ABC):
         """
         if self.history is None:
             raise Exception('No training history - did you train the model yet?')
+        if not isinstance(plot_type, str):
+            raise ValueError('plot_type must be str/string')
 
         fig = None
         if plot_type == 'loss':
@@ -694,7 +695,22 @@ class Learner(ABC):
             plt.ylabel('momentum')
             plt.xlabel('iterations')
         else:
-            raise ValueError('invalid type: choose loss, lr, or momentum')
+            if plot_type not in self.history.history:
+                raise ValueError(f'no {plot_type} in history: are you sure {plot_type} exists in history?')
+            plt.plot(self.history.history[plot_type])
+                
+            val_key = f'val_{plot_type}'
+            if val_key in self.history.history:
+                plt.plot(self.history.history[val_key])
+                legend_items = ['train', 'validation']
+            else:
+                warnings.warn(f'Validation value for {plot_type} wasn\'t found in history')
+                legend_items = ['train']
+                
+            plt.title(f'History of {plot_type}')
+            plt.ylabel(plot_type)
+            plt.xlabel('epoch')
+            plt.legend(legend_items, loc='upper left')
         fig = plt.gcf()
         plt.show()
         if return_fig: return fig
@@ -763,6 +779,7 @@ class Learner(ABC):
         #  use learning_rate schedule
         if cycle_len is not None:
             if not isinstance(callbacks, list): callbacks = []
+            from .lroptimize.sgdr import SGDRScheduler
             schedule = SGDRScheduler(min_lr=min_lr,
                                      max_lr=max_lr,
                                      steps_per_epoch=steps_per_epoch,
@@ -781,7 +798,7 @@ class Learner(ABC):
             if not isinstance(callbacks, list): callbacks = []
             #filepath=os.path.join(folder, "weights-{epoch:02d}-{val_loss:.2f}.hdf5")
             filepath=os.path.join(folder, "weights-{epoch:02d}.hdf5")
-            callbacks.append(ModelCheckpoint(filepath, save_best_only=False, save_weights_only=True))
+            callbacks.append(keras.callbacks.ModelCheckpoint(filepath, save_best_only=False, save_weights_only=True))
         if not callbacks: callbacks=None
         return callbacks
 
@@ -792,7 +809,7 @@ class Learner(ABC):
             if not isinstance(callbacks, list): callbacks = []
             #if StrictVersion(keras.__version__) >= StrictVersion('2.2.3'):
             try:
-                callbacks.append(EarlyStopping(monitor='val_loss', min_delta=0, patience=early_stopping, 
+                callbacks.append(keras.callbacks.EarlyStopping(monitor='val_loss', min_delta=0, patience=early_stopping, 
                                                restore_best_weights=True, verbose=0, mode='auto'))
             except TypeError:
                 warnings.warn("""
@@ -800,7 +817,7 @@ class Learner(ABC):
                               which is only supported on Keras 2.2.3 or greater. 
                               For now, we are falling back to EarlyStopping.restore_best_weights=False.
                               Please use checkpoint_folder option in fit() to restore best weights.""")
-                callbacks.append(EarlyStopping(monitor='val_loss', min_delta=0, patience=early_stopping, 
+                callbacks.append(keras.callbacks.EarlyStopping(monitor='val_loss', min_delta=0, patience=early_stopping, 
                                                verbose=0, mode='auto'))
 
         if not callbacks: callbacks=None
@@ -882,6 +899,8 @@ class Learner(ABC):
         else:
             max_momentum = None
             min_momentum = None
+
+        from .lroptimize.triangular import CyclicLR
         clr = CyclicLR(base_lr=lr/10, max_lr=lr,
                        step_size=math.ceil((steps_per_epoch*epochs)/2), 
                        reduce_on_plateau=0,
@@ -1016,6 +1035,7 @@ class Learner(ABC):
             max_momentum = None
             min_momentum = None
 
+        from .lroptimize.triangular import CyclicLR
         clr = CyclicLR(base_lr=lr/10, max_lr=lr,
                        step_size=step_size, verbose=verbose,
                        monitor=monitor,
@@ -1025,7 +1045,7 @@ class Learner(ABC):
                        min_momentum=min_momentum)
         kcallbacks.append(clr)
         if early_stopping:
-            kcallbacks.append(EarlyStopping(monitor=monitor, min_delta=0, 
+            kcallbacks.append(keras.callbacks.EarlyStopping(monitor=monitor, min_delta=0, 
                                            patience=early_stopping,
                                            restore_best_weights=True, 
                                            verbose=1, mode='auto'))
@@ -1489,8 +1509,8 @@ def get_predictor(model, preproc, batch_size=U.DEFAULT_BS):
     """
 
     # check arguments
-    if not isinstance(model, Model):
-        raise ValueError('model must be of instance Model')
+    if not isinstance(model, keras.Model):
+        raise ValueError('model must be of instance keras.Model')
     if not isinstance(preproc, (ImagePreprocessor,TextPreprocessor, NERPreprocessor, NodePreprocessor, LinkPreprocessor, TabularPreprocessor)):
         raise ValueError('preproc must be instance of ktrain.preprocessor.Preprocessor')
     if isinstance(preproc, ImagePreprocessor):
@@ -1549,17 +1569,21 @@ def load_predictor(fpath, batch_size=U.DEFAULT_BS, custom_objects=None):
     if hasattr(preproc, 'datagen') and hasattr(preproc.datagen, 'ktrain_preproc'):
         preproc_name = preproc.datagen.ktrain_preproc
         if preproc_name == 'resnet50':
-            preproc.datagen.preprocessing_function = pre_resnet50
+            preproc.datagen.preprocessing_function = keras.applications.resnet50.preprocess_input
         elif preproc_name == 'mobilenet':
-            preproc.datagen.preprocessing_function = pre_mobilenet
+            preproc.datagen.preprocessing_function = keras.applications.mobilenet.preprocess_input
+        elif preproc_name == 'mobilenetv3':
+            preproc.datagen.preprocessing_function = keras.applications.mobilenet_v3.preprocess_input
         elif preproc_name == 'inception':
-            preproc.datagen.preprocessing_function = pre_inception
+            preproc.datagen.preprocessing_function = keras.applications.inception_v3.preprocess_input
+        elif preproc_name == 'efficientnet':
+            preproc.datagen.preprocessing_function = keras.applications.efficientnet.preprocess_input
         else:
             raise Exception('Uknown preprocessing_function name: %s' % (preproc_name))
     
     # return the appropriate predictor
-    if not isinstance(model, Model):
-        raise ValueError('model must be of instance Model')
+    if not isinstance(model, keras.Model):
+        raise ValueError('model must be of instance keras.Model')
     if not isinstance(preproc, (ImagePreprocessor, TextPreprocessor, NERPreprocessor, NodePreprocessor, LinkPreprocessor, TabularPreprocessor)):
         raise ValueError('preproc must be instance of ktrain.preprocessor.Preprocessor')
     if isinstance(preproc, ImagePreprocessor):
@@ -1630,8 +1654,26 @@ def _load_model(fpath, preproc=None, train_data=None, custom_objects=None):
                     type(preproc).__name__ == 'NERPreprocessor')) or \
         train_data and U.is_ner(data=train_data):
         from .text.ner.anago.layers import CRF
-        from .text.ner import crf_loss
+        from .text.ner.anago.layers import crf_loss
         custom_objects={'CRF': CRF, 'crf_loss':crf_loss}
+        # save old te_model as backup
+        if preproc:
+            old_te_model = preproc.p.te_model
+            # load TransformerEmbedding model from fpath/hf folder
+            # if model_name is local_path, update it to reflect current predictor folder, since
+            # all model/tokenizer/config files should have been saved there by predictor.save
+            
+            preproc.p.te_model = os.path.join(fpath, 'hf') if preproc.p.te_model else preproc.p.te_model
+            if preproc.p.te_model:
+                # te_model should point fpath/hf folder
+                try:
+                    preproc.p.activate_transformer(preproc.p.te_model, layers=preproc.p.te_layers)
+                except:
+                    # fall back to old model id or location if error for backwards compatibility
+                    warnings.warn(f'could not load TransformerEmbedding model from {preproc.p.te_model} - trying {old_te_model}')
+                    preproc.p.te_model = old_te_model
+                    preproc.p.activate_transformer(preproc.p.te_model, layers=preproc.p.te_layers)
+
     elif (preproc and (isinstance(preproc, NodePreprocessor) or \
                     type(preproc).__name__ == 'NodePreprocessor')) or \
         train_data and U.is_nodeclass(data=train_data):
@@ -1643,18 +1685,19 @@ def _load_model(fpath, preproc=None, train_data=None, custom_objects=None):
         from stellargraph.layer import MeanAggregator
         custom_objects={'MeanAggregator': MeanAggregator}
     custom_objects = {} if custom_objects is None else custom_objects
+    from .lroptimize.optimization import AdamWeightDecay
     custom_objects['AdamWeightDecay'] = AdamWeightDecay
     try:
         try:
-            model = load_model(os.path.join(fpath, U.MODEL_NAME), custom_objects=custom_objects)
+            model = keras.models.load_model(os.path.join(fpath, U.MODEL_NAME), custom_objects=custom_objects)
         except:
             try:
                 # pre-0.16: model fpath was file name of model not folder for non-Transformer models
                 #warnings.warn('could not load model as %s - attempting to load model as %s' % (os.path.join(fpath, U.MODEL_NAME), fpath))
-                model = load_model(fpath, custom_objects=custom_objects)
+                model = keras.models.load_model(fpath, custom_objects=custom_objects)
             except:
                 # for bilstm models without CRF layer on TF2 where CRF is not supported 
-                model = load_model(fpath, custom_objects={'AdamWeightDecay':AdamWeightDecay})
+                model = keras.models.load_model(fpath, custom_objects={'AdamWeightDecay':AdamWeightDecay})
     except Exception as e:
         print('Call to keras.models.load_model failed. Try manually invoking this function to investigate error and report issue if necessary.')
         raise Exception('Error detected: %s' % (e))
