@@ -110,7 +110,7 @@ class LRFinder:
         # Set the initial learning rate
         K.set_value(self.model.optimizer.lr, start_lr)
 
-        callback = LambdaCallback(on_batch_end=lambda batch, logs: self.on_batch_end(batch, logs))
+        callback = keras.callbacks.LambdaCallback(on_batch_end=lambda batch, logs: self.on_batch_end(batch, logs))
 
 
         if use_gen:
@@ -171,18 +171,46 @@ class LRFinder:
                 print("Failed to compute the gradients, there might not be enough points.\n" +\
                        "Plot displayed without suggestion.")
             else:
+                valley = self.valley(self.lrs, self.losses)
                 mg = self.mg
                 ml = self.ml
-                print('Two possible suggestions for LR from plot:')
-                print(f"\tMin numerical gradient: {self.lrs[mg]:.2E}")
-                print(f"\tMin loss divided by 10: {self.lrs[ml]/10:.2E}")
-                ax.plot(self.lrs[mg],self.losses[mg], markersize=10,marker='o',color='red')
+                print('Three possible suggestions for LR from plot:')
+                print(f"\tLongest valley (red): {self.lrs[valley]:.2E}")
+                print(f"\tMin numerical gradient (purple): {self.lrs[mg]:.2E}")
+                print(f"\tMin loss divided by 10 (omitted from plot): {self.lrs[ml]/10:.2E}")
+                ax.plot(self.lrs[valley],self.losses[valley], markersize=10,marker='o',color='red')
+                ax.plot(self.lrs[mg],self.losses[mg], markersize=10,marker='o',color='purple')
         fig = plt.gcf()
         plt.show()
         if return_fig: return fig
         return
 
 
+    def valley(self, lrs, losses):
+        """
+        valley method for LR suggestions:
+        https://github.com/fastai/fastai/pull/3377
+        """
+        n = len(losses)
+        max_start, max_end = 0,0
+
+        # find the longest valley
+        lds = [1]*n
+        for i in range(1,n):
+            for j in range(0,i):
+                if (losses[i] < losses[j]) and (lds[i] < lds[j] + 1):
+                    lds[i] = lds[j] + 1
+                if lds[max_end] < lds[i]:
+                    max_end = i
+                    max_start = max_end - lds[max_end]
+
+        sections = (max_end - max_start) / 3
+        idx = max_start + int(sections) + int(sections/2)
+
+        #return lrs[idx], (lrs[idx], losses[idx])
+        return idx
+
+
     def _compute_stats(self):
         """
         ```
@@ -195,8 +223,9 @@ class LRFinder:
         self.ml = np.argmin(self.losses)
         try: 
             self.mg = (np.gradient(np.array(self.losses[32:self.ml]))).argmin()
-        except:
+        except Exception as e:
             self.mg = None
+            warnings.warn(str(e))
         return
 
 
@@ -206,6 +235,7 @@ class LRFinder:
         Generates two numerical estimates of lr: 
           1. lr associated with minum numerical gradient (None if gradient computation fails)
           2. lr associated with minimum loss divided by 10
+          3. lr associated with longest valley
         Args:
           tuple: (float, float)
 
@@ -218,7 +248,9 @@ class LRFinder:
         if self.mg is not None:
             lr1 = self.lrs[self.mg]
         lr2 = self.lrs[self.ml]/10
-        return (lr1, lr2)
+        lr3 = self.lrs[self.valley(self.lrs, self.losses)]
+
+        return (lr1, lr2, lr3)
 
 
     def find_called(self):
