@@ -2,7 +2,7 @@ from __future__ import absolute_import
 from __future__ import division
 from ....imports import *
 
-class CRF(Layer):
+class CRF(keras.layers.Layer):
     """An implementation of linear chain conditional random field (CRF).
     An linear chain CRF is defined to maximize the following likelihood function:
     $$ L(W, U, b; y_1, ..., y_n) := \frac{1}{Z} \sum_{y_1, ..., y_n} \exp(-a_1' y_1 - a_n' y_n
@@ -142,17 +142,17 @@ class CRF(Layer):
         self.use_boundary = use_boundary
         self.use_bias = use_bias
 
-        self.activation = activations.get(activation)
+        self.activation = keras.activations.get(activation)
 
-        self.kernel_initializer = initializers.get(kernel_initializer)
-        self.chain_initializer = initializers.get(chain_initializer)
-        self.boundary_initializer = initializers.get(boundary_initializer)
-        self.bias_initializer = initializers.get(bias_initializer)
+        self.kernel_initializer = keras.initializers.get(kernel_initializer)
+        self.chain_initializer = keras.initializers.get(chain_initializer)
+        self.boundary_initializer = keras.initializers.get(boundary_initializer)
+        self.bias_initializer = keras.initializers.get(bias_initializer)
 
-        self.kernel_regularizer = regularizers.get(kernel_regularizer)
-        self.chain_regularizer = regularizers.get(chain_regularizer)
-        self.boundary_regularizer = regularizers.get(boundary_regularizer)
-        self.bias_regularizer = regularizers.get(bias_regularizer)
+        self.kernel_regularizer = keras.regularizers.get(kernel_regularizer)
+        self.chain_regularizer = keras.regularizers.get(chain_regularizer)
+        self.boundary_regularizer = keras.regularizers.get(boundary_regularizer)
+        self.bias_regularizer = keras.regularizers.get(bias_regularizer)
 
         self.kernel_constraint = constraints.get(kernel_constraint)
         self.chain_constraint = constraints.get(chain_constraint)
@@ -162,7 +162,7 @@ class CRF(Layer):
         self.unroll = unroll
 
     def build(self, input_shape):
-        self.input_spec = [InputSpec(shape=input_shape)]
+        self.input_spec = [keras.layers.InputSpec(shape=input_shape)]
         self.input_dim = input_shape[-1]
 
         self.kernel = self.add_weight((self.input_dim, self.units),
@@ -233,15 +233,15 @@ class CRF(Layer):
                   'use_boundary': self.use_boundary,
                   'use_bias': self.use_bias,
                   'sparse_target': self.sparse_target,
-                  'kernel_initializer': initializers.serialize(self.kernel_initializer),
-                  'chain_initializer': initializers.serialize(self.chain_initializer),
-                  'boundary_initializer': initializers.serialize(self.boundary_initializer),
-                  'bias_initializer': initializers.serialize(self.bias_initializer),
-                  'activation': activations.serialize(self.activation),
-                  'kernel_regularizer': regularizers.serialize(self.kernel_regularizer),
-                  'chain_regularizer': regularizers.serialize(self.chain_regularizer),
-                  'boundary_regularizer': regularizers.serialize(self.boundary_regularizer),
-                  'bias_regularizer': regularizers.serialize(self.bias_regularizer),
+                  'kernel_initializer': keras.initializers.serialize(self.kernel_initializer),
+                  'chain_initializer': keras.initializers.serialize(self.chain_initializer),
+                  'boundary_initializer': keras.initializers.serialize(self.boundary_initializer),
+                  'bias_initializer': keras.initializers.serialize(self.bias_initializer),
+                  'activation': keras.activations.serialize(self.activation),
+                  'kernel_regularizer': keras.regularizers.serialize(self.kernel_regularizer),
+                  'chain_regularizer': keras.regularizers.serialize(self.chain_regularizer),
+                  'boundary_regularizer': keras.regularizers.serialize(self.boundary_regularizer),
+                  'bias_regularizer': keras.regularizers.serialize(self.bias_regularizer),
                   'kernel_constraint': constraints.serialize(self.kernel_constraint),
                   'chain_constraint': constraints.serialize(self.chain_constraint),
                   'boundary_constraint': constraints.serialize(self.boundary_constraint),
@@ -266,9 +266,9 @@ class CRF(Layer):
             return loss
         else:
             if self.sparse_target:
-                return sparse_categorical_crossentropy
+                return keras.losses.sparse_categorical_crossentropy
             else:
-                return categorical_crossentropy
+                return keras.losses.categorical_crossentropy
 
     @property
     def accuracy(self):
@@ -548,6 +548,6 @@ def crf_loss(y_true, y_pred):
         return crf_nll(y_true, y_pred)
     else:
         if crf.sparse_target:
-            return sparse_categorical_crossentropy(y_true, y_pred)
+            return keras.losses.sparse_categorical_crossentropy(y_true, y_pred)
         else:
-            return categorical_crossentropy(y_true, y_pred)
+            return keras.losses.categorical_crossentropy(y_true, y_pred)
