@@ -1,7 +1,6 @@
 from ..imports import *
 from .. import utils as U
 from ..preprocessor import Preprocessor
-from ..data import SequenceDataset
 from . import textutils as TU
 
 from transformers import AutoConfig, TFAutoModelForSequenceClassification, AutoTokenizer, TFAutoModel
@@ -163,6 +162,8 @@ def bert_tokenize(docs, tokenizer, max_length, verbose=1):
     indices = []
     for i in mb:
         for doc in pb:
+            # https://stackoverflow.com/questions/67360987/bert-model-bug-encountered-during-training/67375675#67375675
+            doc = str(doc) if isinstance(doc, (float, int)) else doc
             ids, segments = tokenizer.encode(doc, max_len=max_length)
             indices.append(ids)
         if verbose: mb.write('done.')
@@ -384,6 +385,7 @@ def hf_convert_examples(texts, y=None, tokenizer=None,
     # due to issues in transormers library and TF2 tf.Datasets, arrays are converted
     # to iterators on-the-fly
     #return  TransformerSequence(np.array(features_list), np.array(labels))
+    from .dataset import TransformerDataset
     return  TransformerDataset(np.array(features_list), np.array(labels))
 
 
@@ -610,7 +612,7 @@ class StandardTextPreprocessor(TextPreprocessor):
         train_text = self.process_chinese(train_text, lang=self.lang)
 
         # extract vocabulary
-        self.tok = Tokenizer(num_words=self.max_features)
+        self.tok = keras.preprocessing.text.Tokenizer(num_words=self.max_features)
         self.tok.fit_on_texts(train_text)
         U.vprint('Word Counts: {}'.format(len(self.tok.word_counts)), verbose=verbose)
         U.vprint('Nrows: {}'.format(len(train_text)), verbose=verbose)
@@ -624,7 +626,7 @@ class StandardTextPreprocessor(TextPreprocessor):
         x_train = self._fit_ngrams(x_train, verbose=verbose)
 
         # pad sequences
-        x_train = sequence.pad_sequences(x_train, maxlen=self.maxlen)
+        x_train = keras.preprocessing.sequence.pad_sequences(x_train, maxlen=self.maxlen)
         U.vprint('x_train shape: ({},{})'.format(x_train.shape[0], x_train.shape[1]), verbose=verbose)
 
         # transform y
@@ -662,7 +664,7 @@ class StandardTextPreprocessor(TextPreprocessor):
 
 
         # pad sequences
-        x_test = sequence.pad_sequences(x_test, maxlen=self.maxlen)
+        x_test = keras.preprocessing.sequence.pad_sequences(x_test, maxlen=self.maxlen)
         U.vprint('x_test shape: ({},{})'.format(x_test.shape[0], x_test.shape[1]), verbose=verbose)
 
         # transform y
@@ -772,6 +774,7 @@ class BERTPreprocessor(TextPreprocessor):
             for line in reader:
                 token = line.strip()
                 token_dict[token] = len(token_dict)
+        from keras_bert import Tokenizer as BERT_Tokenizer
         tokenizer = BERT_Tokenizer(token_dict)
         self.tok = tokenizer
         self.tok_dct = dict((v,k) for k,v in token_dict.items())
@@ -1074,6 +1077,8 @@ class TransformersPreprocessor(TextPreprocessor):
                     #raise ValueError('could not load pretrained model %s using both from_pt=False and from_pt=True' % (mname))
         else:
             model = self.model_type.from_pretrained(mname, num_labels=num_labels)
+        # ISSUE 416: mname is either model name (e.g., bert-base-uncased) or path to folder with tokenizer files
+        self.get_tokenizer(mname)
         return model
 
 
@@ -1435,89 +1440,89 @@ class TransformerEmbedding():
         return np.array(embeddings)
 
 
-class TransformerDataset(SequenceDataset):
-    """
-    ```
-    Wrapper for Transformer datasets.
-    ```
-    """
-
-    def __init__(self, x, y, batch_size=1):
-        if type(x) not in [list, np.ndarray]: raise ValueError('x must be list or np.ndarray')
-        if type(y) not in [list, np.ndarray]: raise ValueError('y must be list or np.ndarray')
-        if type(x) == list: x = np.array(x)
-        if type(y) == list: y = np.array(y)
-        self.x = x
-        self.y = y
-        self.batch_size = batch_size
 
 
-    def __getitem__(self, idx):
-        batch_x = self.x[idx * self.batch_size: (idx + 1) * self.batch_size]
-        batch_y = self.y[idx * self.batch_size: (idx + 1) * self.batch_size]
-        return (batch_x, batch_y)
 
 
-    def __len__(self):
-        return math.ceil(len(self.x) / self.batch_size)
 
 
-    def to_tfdataset(self, train=True):
-        """
-        ```
-        convert transformer features to tf.Dataset
-        ```
-        """
-        if train:
-            shuffle=True
-            repeat = True
-        else:
-            shuffle=False
-            repeat=False
-
-        if len(self.y.shape) == 1:
-            yshape = []
-            ytype = tf.float32
-        else:
-            yshape = [None]
-            ytype = tf.int64
-
-        def gen():
-            for idx, data in enumerate(self.x):
-                yield ({'input_ids': data[0],
-                         'attention_mask': data[1],
-                         'token_type_ids': data[2]},
-                        self.y[idx])
-
-        tfdataset= tf.data.Dataset.from_generator(gen,
-            ({'input_ids': tf.int32,
-              'attention_mask': tf.int32,
-              'token_type_ids': tf.int32},
-             ytype),
-            ({'input_ids': tf.TensorShape([None]),
-              'attention_mask': tf.TensorShape([None]),
-              'token_type_ids': tf.TensorShape([None])},
-             tf.TensorShape(yshape)))
-
-        if shuffle:
-            tfdataset = tfdataset.shuffle(self.x.shape[0])
-        tfdataset = tfdataset.batch(self.batch_size)
-        if repeat:
-            tfdataset = tfdataset.repeat(-1)
-        return tfdataset
 
 
-    def get_y(self):
-        return self.y
 
-    def nsamples(self):
-        return len(self.x)
 
-    def nclasses(self):
-        return self.y.shape[1]
 
-    def xshape(self):
-        return (len(self.x), self.x[0].shape[1])
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 
 # preprocessors
