@@ -1,5 +1,4 @@
 from .imports import *
-from .data import Dataset
 
 
 #------------------------------------------------------------------------------
@@ -42,6 +41,11 @@ PREPROC_NAME = MODEL_BASENAME+'.preproc'
 # DATA/MODEL INSPECTORS
 #------------------------------------------------------------------------------
 
+def is_ktrain_dataset(data):
+    from .dataset import Dataset
+    return isinstance(data, Dataset)
+
+
 def loss_fn_from_model(model):
     # dep_fix
     if version.parse(tf.__version__) < version.parse('2.2'):
@@ -116,8 +120,8 @@ def is_classifier(model):
             last = model.layers[-1]
             output_shape = last.output_shape
             mult_output = True if len(output_shape) ==2 and output_shape[1] >  1 else False
-            if ( (hasattr(last, 'activation') and isinstance(last.activation, type(sigmoid))) or\
-               isinstance(last, type(sigmoid)) ) and mult_output:
+            if ( (hasattr(last, 'activation') and isinstance(last.activation, type(keras.activations.sigmoid))) or\
+               isinstance(last, type(keras.activations.sigmoid)) ) and mult_output:
                 is_multilabel = True
     return (is_classifier, is_multilabel)
 
@@ -250,7 +254,7 @@ def is_multilabel(data):
 def shape_from_data(data):
     err_msg = 'could not determine shape from %s' % (type(data))
     if is_iter(data):
-        if isinstance(data, Dataset): return data.xshape()
+        if is_ktrain_dataset(data): return data.xshape()
         elif hasattr(data, 'image_shape'): return data.image_shape          # DirectoryIterator/DataFrameIterator
         elif hasattr(data, 'x'):                                            # NumpyIterator
             return data.x.shape[1:]
@@ -280,7 +284,7 @@ def ondisk(data):
 def nsamples_from_data(data):
     err_msg = 'could not determine number of samples from %s' % (type(data))
     if is_iter(data):
-        if isinstance(data, Dataset): return data.nsamples()
+        if is_ktrain_dataset(data): return data.nsamples()
         elif hasattr(data, 'samples'):  # DirectoryIterator/DataFrameIterator
             return data.samples
         elif hasattr(data, 'n'):     # DirectoryIterator/DataFrameIterator/NumpyIterator
@@ -299,7 +303,7 @@ def nsamples_from_data(data):
 
 def nclasses_from_data(data):
     if is_iter(data):
-        if isinstance(data, Dataset): return data.nclasses()
+        if is_ktrain_dataset(data): return data.nclasses()
         elif hasattr(data, 'classes'):   # DirectoryIterator
             return len(set(data.classes))
         else:
@@ -316,9 +320,9 @@ def nclasses_from_data(data):
 
 def y_from_data(data):
     if is_iter(data):
-        if isinstance(data, Dataset): return data.get_y()
+        if is_ktrain_dataset(data): return data.get_y()
         elif hasattr(data, 'classes'): # DirectoryIterator
-            return to_categorical(data.classes)
+            return keras.utils.to_categorical(data.classes)
         elif hasattr(data, 'labels'):  # DataFrameIterator
             return data.labels
         elif hasattr(data, 'y'): # NumpyArrayIterator
@@ -336,7 +340,7 @@ def y_from_data(data):
 def is_iter(data, ignore=False):
     if ignore: return True
     iter_classes = ["NumpyArrayIterator", "DirectoryIterator", "DataFrameIterator"]
-    return data.__class__.__name__ in iter_classes or isinstance(data, Dataset)
+    return data.__class__.__name__ in iter_classes or is_ktrain_dataset(data)
 
 
 
@@ -477,6 +481,27 @@ def get_ktrain_data():
 #------------------------------------------------------------------------------
 # MISC UTILITIES
 #------------------------------------------------------------------------------
+
+def batchify(X, size):
+    """
+    ```
+    Splits X into separate batch sizes specified by size.
+    Args:
+        X(list): elements
+        size(int): batch size
+    Returns:
+        list of evenly sized batches with the last batch having the remaining elements
+    ```
+    """
+    return [X[x : x + size] for x in range(0, len(X), size)]
+
+
+def list2chunks(a, n):
+    k, m = divmod(len(a), n)
+    return (a[i * k + min(i, m):(i + 1) * k + min(i + 1, m)] for i in range(n))
+
+
+
 def check_array(X, y=None, X_name='X', y_name='targets' ):
     if not isinstance(X, (list, np.ndarray)): raise ValueError("%s must be a list or NumPy array" % X_name)
     if y is not None and not isinstance(y, (list, np.ndarray)): raise ValueError("%s must be a list or NumPy array" % y_name)
@@ -523,9 +548,9 @@ def get_random_colors(n, name='hsv', hex_format=True):
     return np.array(result)
 
 
-def list2chunks(a, n):
-    k, m = divmod(len(a), n)
-    return (a[i * k + min(i, m):(i + 1) * k + min(i + 1, m)] for i in range(n))
+
+
+
 
 
 def get_hf_model_name(model_id):
@@ -617,7 +642,7 @@ class YTransform:
 
             if train and ( len(set(targets)) != int(max(targets)+1) ):
                 raise ValueError('len(set(targets) is %s but max(targets)+1 is  %s' % ( len(set(targets)), int(max(targets)+1) ))
-            targets = to_categorical(targets, num_classes=len(self.get_classes()))
+            targets = keras.utils.to_categorical(targets, num_classes=len(self.get_classes()))
         if train: self.train_called=True
         return targets
 
