@@ -7,21 +7,38 @@ from .wrn import create_wide_residual_network
 
 PRETRAINED_RESNET50 = 'pretrained_resnet50'
 PRETRAINED_MOBILENET = 'pretrained_mobilenet'
+PRETRAINED_MOBILENETV3 = 'pretrained_mobilenetv3'
 PRETRAINED_INCEPTION = 'pretrained_inception'
+PRETRAINED_EFFICIENTNETB1 = 'pretrained_efficientnetb1'
+PRETRAINED_EFFICIENTNETB7 = 'pretrained_efficientnetb7'
 RESNET50 = 'resnet50'
 MOBILENET = 'mobilenet'
+MOBILENETV3 = 'mobilenetv3'
 INCEPTION = 'inception'
+EFFICIENTNETB1 = 'efficientnetb1'
+EFFICIENTNETB7 = 'efficientnetb7'
 CNN = 'default_cnn'
 WRN22 = 'wrn22'
-PRETRAINED_MODELS = [PRETRAINED_RESNET50, PRETRAINED_MOBILENET, PRETRAINED_INCEPTION]
-PREDEFINED_MODELS = PRETRAINED_MODELS + [RESNET50, MOBILENET, INCEPTION]
+PRETRAINED_MODELS = [
+    PRETRAINED_RESNET50, PRETRAINED_MOBILENET, PRETRAINED_MOBILENETV3,
+    PRETRAINED_INCEPTION, PRETRAINED_EFFICIENTNETB1, PRETRAINED_EFFICIENTNETB7
+]
+PREDEFINED_MODELS = PRETRAINED_MODELS + [
+    RESNET50, MOBILENET, MOBILENETV3, INCEPTION, EFFICIENTNETB1, EFFICIENTNETB7
+]
 IMAGE_CLASSIFIERS = {
                      PRETRAINED_RESNET50: '50-layer Residual Network (pretrained on ImageNet)',
-                     RESNET50:  '50-layer Resididual Network (randomly initialized)',
+                     RESNET50:  '50-layer Resididual Network (randomly initialized) [https://arxiv.org/abs/1512.03385]',
                      PRETRAINED_MOBILENET: 'MobileNet Neural Network (pretrained on ImageNet)',
-                     MOBILENET:  'MobileNet Neural Network (randomly initialized)',
+                     MOBILENET:  'MobileNet Neural Network (randomly initialized) [https://arxiv.org/abs/1704.04861]',
+                     PRETRAINED_MOBILENETV3: 'MobileNetV3-Small Neural Network (pretrained on ImageNet)',
+                     MOBILENETV3:  'MobileNetV3-Small Neural Network (randomly initialized) [https://arxiv.org/abs/1905.02244]',
                      PRETRAINED_INCEPTION: 'Inception Version 3  (pretrained on ImageNet)',
-                     INCEPTION:  'Inception Version 3 (randomly initialized)',
+                     INCEPTION:  'Inception Version 3 (randomly initialized) [http://arxiv.org/abs/1512.00567]',
+                     PRETRAINED_EFFICIENTNETB1: 'EfficientNet-B1 Neural Network (pretrained on ImageNet)',
+                     EFFICIENTNETB1: 'EfficientNet-B1 Neural Network (pretrained on ImageNet) [https://arxiv.org/abs/1905.11946]',
+                     PRETRAINED_EFFICIENTNETB7: 'EfficientNet-B7 Neural Network (pretrained on ImageNet)',
+                     EFFICIENTNETB7: 'EfficientNet-B7 Neural Network (pretrained on ImageNet) [https://arxiv.org/abs/1905.11946]',
                      WRN22: '22-layer Wide Residual Network (randomly initialized)',
                      CNN : 'a default LeNet-like Convolutional Neural Network'}
 
@@ -39,7 +56,7 @@ def pretrained_datagen(data, name):
     if not data or not U.is_iter(data): return
     idg = data.image_data_generator
     if name == PRETRAINED_RESNET50:
-        idg.preprocessing_function = pre_resnet50
+        idg.preprocessing_function = keras.applications.resnet50.preprocess_input
         idg.ktrain_preproc = 'resnet50'
         idg.rescale=None
         idg.featurewise_center=False
@@ -48,7 +65,7 @@ def pretrained_datagen(data, name):
         idg.samplewise_std_normalization=False
         idg.zca_whitening = False
     elif name == PRETRAINED_MOBILENET:
-        idg.preprocessing_function = pre_mobilenet
+        idg.preprocessing_function = keras.applications.mobilenet.preprocess_input
         idg.ktrain_preproc = 'mobilenet'
         idg.rescale=None
         idg.featurewise_center=False
@@ -56,8 +73,17 @@ def pretrained_datagen(data, name):
         idg.featurewise_std_normalization=False
         idg.samplewise_std_normalization=False
         idg.zca_whitening = False
+    elif name == PRETRAINED_MOBILENETV3:
+        idg.preprocessing_function = keras.applications.mobilenet_v3.preprocess_input
+        idg.ktrain_preproc = 'mobilenetv3'
+        idg.rescale=None
+        idg.featurewise_center=False
+        idg.samplewise_center=False
+        idg.featurewise_std_normalization=False
+        idg.samplewise_std_normalization=False
+        idg.zca_whitening = False
     elif name == PRETRAINED_INCEPTION:
-        idg.preprocessing_function = pre_inception
+        idg.preprocessing_function = keras.applications.inception_v3.preprocess_input
         idg.ktrain_preproc = 'inception'
         idg.rescale=None
         idg.featurewise_center=False
@@ -65,6 +91,15 @@ def pretrained_datagen(data, name):
         idg.featurewise_std_normalization=False
         idg.samplewise_std_normalization=False
         idg.zca_whitening = False
+    elif name == PRETRAINED_EFFICIENTNETB1 or name == PRETRAINED_EFFICIENTNETB7:
+        idg.preprocessing_function = keras.applications.efficientnet.preprocess_input
+        idg.ktrain_preproc = 'efficientnet'
+        idg.rescale=None
+        idg.featurewise_center=False
+        idg.samplewise_center=False
+        idg.featurewise_std_normalization=False
+        idg.samplewise_std_normalization=False
+        idg.zca_whitening = False
 
     return
 
@@ -83,13 +118,13 @@ def image_classifier(name,
 
     """
     ```
-    Returns a pre-trained ResNet50 model ready to be fine-tuned
+    Returns a pre-defined/pre-trained model ready to be trained/fine-tuned
     for multi-class classification. By default, all layers are
     trainable/unfrozen.
 
 
     Args:
-        name (string): one of {'pretrained_resnet50', 'resnet50', 'default_cnn'}
+        name (string): one of model shown on ktrain.vision.print_image_classifiers
         train_data (image.Iterator): train data. Note: Will be manipulated here!
         val_data (image.Iterator): validation data.  Note: Will be manipulated here!
         freeze_layers (int):  number of beginning layers to make untrainable
@@ -132,13 +167,13 @@ def image_regression_model(name,
 
     """
     ```
-    Returns a pre-trained ResNet50 model ready to be fine-tuned
+    Returns a pre-defined/pre-trained model ready to be trained/fine-tuned
     for multi-class classification. By default, all layers are
     trainable/unfrozen.
 
 
     Args:
-        name (string): one of {'pretrained_resnet50', 'resnet50', 'default_cnn'}
+        name (string): one of model shown on ktrain.vision.print_image_regression_models
         train_data (image.Iterator): train data. Note: Will be manipulated here!
         val_data (image.Iterator): validation data.  Note: Will be manipulated here!
         freeze_layers (int):  number of beginning layers to make untrainable
@@ -183,13 +218,13 @@ def image_model( name,
 
     """
     ```
-    Returns a pre-trained ResNet50 model ready to be fine-tuned
+    Returns a pre-defined/pre-trained model ready to be trained/fine-tuned
     for multi-class classification or regression. By default, all layers are
     trainable/unfrozen.
 
 
     Args:
-        name (string): one of {'pretrained_resnet50', 'resnet50', 'default_cnn'}
+        name (string): one of model shown on ktrain.vision.print_image_classifiers
         train_data (image.Iterator): train data. Note: Will be manipulated here!
         val_data (image.Iterator): validation data.  Note: Will be manipulated here!
         freeze_layers (int):  number of beginning layers to make untrainable
@@ -225,6 +260,9 @@ def image_model( name,
                          'NumpyArrayIterator) - please use the ktrain.data.images_from* ' +\
                          'functions')
 
+    # check for MobileNetV3
+    if name in [PRETRAINED_MOBILENETV3, MOBILENETV3] and not HAS_MOBILENETV3:
+        raise ValueError(f'You chose {name}, but it does not appear to be available in your version of TensorFlow.')
 
 
     # set pretrained flag
@@ -322,27 +360,27 @@ def build_visionmodel(name,
 def build_cnn(num_classes, 
               input_shape=(28,28,1),
               activation='softmax'):
-    model = Sequential()
-    model.add(Conv2D(32, kernel_size=(3, 3),activation='relu',
+    model = keras.models.Sequential()
+    model.add(keras.layers.Conv2D(32, kernel_size=(3, 3),activation='relu',
                      kernel_initializer='he_normal',input_shape=input_shape))
-    model.add(Conv2D(32, kernel_size=(3, 3),activation='relu',
+    model.add(keras.layers.Conv2D(32, kernel_size=(3, 3),activation='relu',
                      kernel_initializer='he_normal'))
-    model.add(MaxPooling2D((2, 2)))
-    model.add(Dropout(0.20))
-    model.add(Conv2D(64, (3, 3), activation='relu',padding='same',
+    model.add(keras.layers.MaxPooling2D((2, 2)))
+    model.add(keras.layers.Dropout(0.20))
+    model.add(keras.layers.Conv2D(64, (3, 3), activation='relu',padding='same',
                       kernel_initializer='he_normal'))
-    model.add(Conv2D(64, (3, 3), activation='relu',padding='same',
+    model.add(keras.layers.Conv2D(64, (3, 3), activation='relu',padding='same',
                      kernel_initializer='he_normal'))
-    model.add(MaxPooling2D(pool_size=(2, 2)))
-    model.add(Dropout(0.25))
-    model.add(Conv2D(128, (3, 3), activation='relu',padding='same',
+    model.add(keras.layers.MaxPooling2D(pool_size=(2, 2)))
+    model.add(keras.layers.Dropout(0.25))
+    model.add(keras.layers.Conv2D(128, (3, 3), activation='relu',padding='same',
                      kernel_initializer='he_normal'))
-    model.add(Dropout(0.25))
-    model.add(Flatten())
-    model.add(Dense(128, activation='relu'))
-    model.add(BatchNormalization())
-    model.add(Dropout(0.25))
-    model.add(Dense(num_classes, activation=activation))
+    model.add(keras.layers.Dropout(0.25))
+    model.add(keras.layers.Flatten())
+    model.add(keras.layers.Dense(128, activation='relu'))
+    model.add(keras.layers.BatchNormalization())
+    model.add(keras.layers.Dropout(0.25))
+    model.add(keras.layers.Dense(num_classes, activation=activation))
     return model
 
 
@@ -387,20 +425,35 @@ def build_predefined(
     if name in [RESNET50, PRETRAINED_RESNET50]:
         with warnings.catch_warnings():
             warnings.simplefilter('ignore')
-            net = ResNet50(include_top=include_top, 
+            net = keras.applications.ResNet50(include_top=include_top, 
                            weights=weights,
                            input_tensor=input_tensor,
                            input_shape = input_shape)
     elif name in [MOBILENET, PRETRAINED_MOBILENET]:
-        net = MobileNet(include_top=include_top, 
+        net = keras.applications.MobileNet(include_top=include_top, 
                         weights=weights,
                         input_tensor=input_tensor,
                         input_shape = input_shape)
+    elif name in [MOBILENETV3, PRETRAINED_MOBILENETV3]:
+        net = keras.applications.MobileNetV3Small(include_top=include_top, 
+                               weights=weights,
+                               input_tensor=input_tensor,
+                               input_shape = input_shape)
     elif name in [INCEPTION, PRETRAINED_INCEPTION]:
-        net = InceptionV3(include_top=include_top, 
+        net = keras.applications.InceptionV3(include_top=include_top, 
                           weights=weights,
                           input_tensor=input_tensor,
                            input_shape = input_shape)
+    elif name in [EFFICIENTNETB1, PRETRAINED_EFFICIENTNETB1]:
+        net = keras.applications.EfficientNetB1(include_top=include_top, 
+                          weights=weights,
+                          input_tensor=input_tensor,
+                          input_shape = input_shape)
+    elif name in [EFFICIENTNETB7, PRETRAINED_EFFICIENTNETB7]:
+        net = keras.applications.EfficientNetB7(include_top=include_top, 
+                          weights=weights,
+                          input_tensor=input_tensor,
+                          input_shape = input_shape)
     else:
         raise ValueError('Unsupported model: %s' % (name))
 
@@ -410,7 +463,7 @@ def build_predefined(
             layer.trainable = False
 
     x = net.output
-    x = Flatten()(x)
+    x = keras.layers.Flatten()(x)
 
     # xtra FCs in pretrained model
     if name in PRETRAINED_MODELS:
@@ -420,15 +473,15 @@ def build_predefined(
             p = pt_ps[i]
             fc_name = "fc%s" % (i)
             if p is not None:
-                x = Dropout(p)(x)
-            x = Dense(fc, activation='relu', 
+                x = keras.layers.Dropout(p)(x)
+            x = keras.layers.Dense(fc, activation='relu', 
                       kernel_initializer='he_normal', name=fc_name)(x)
 
 
     # final FC
-    x = Dropout(dropout)(x)
-    output_layer = Dense(num_classes, activation=activation, name=activation)(x)
-    model = Model(inputs=net.input, outputs=output_layer)
+    x = keras.layers.Dropout(dropout)(x)
+    output_layer = keras.layers.Dense(num_classes, activation=activation, name=activation)(x)
+    model = keras.Model(inputs=net.input, outputs=output_layer)
 
     if freeze_layers is not None:
         # set certain earlier layers as non-trainable
